@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2002-2004, Index Data.
+ * Copyright (c) 2002-2005, Index Data.
  * See the file LICENSE for details.
  *
- * $Id: srw.h,v 1.17 2004/10/15 00:18:59 adam Exp $
+ * $Id: srw.h,v 1.19 2005/01/09 21:52:48 adam Exp $
  */
 /**
  * \file srw.h
@@ -90,7 +90,12 @@ typedef struct {
 } Z_SRW_explainResponse;
     
 typedef struct {
-    char *scanClause;
+    int query_type;
+    union {
+        char *cql;
+        char *xcql;
+        char *pqf;
+    } scanClause;
     int *responsePosition;
     int *maximumTerms;
     char *stylesheet;
@@ -101,6 +106,7 @@ typedef struct {
     char *value;
     int *numberOfRecords;
     char *displayTerm;
+    char *whereInList;
 } Z_SRW_scanTerm;
 
 typedef struct {
