@@ -1,10 +1,15 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: odr_bit.c,v 1.1 2003/10/27 12:21:33 adam Exp $
+ * $Id: odr_bit.c,v 1.3 2004/10/15 00:19:00 adam Exp $
  */
+
+/**
+ * \file odr_bit.c
+ * \brief Implements ODR BITSTRING codec
+ */
+
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -34,7 +39,7 @@ int odr_bitstring(ODR o, Odr_bitmask **p, int opt, const char *name)
     if (o->direction == ODR_PRINT)
     {
 	odr_prname(o, name);
-    	fprintf(o->print, "BITSTRING(len=%d)\n",(*p)->top + 1);
+    	odr_printf(o, "BITSTRING(len=%d)\n",(*p)->top + 1);
     	return 1;
     }
     if (o->direction == ODR_DECODE)
