@@ -1,9 +1,12 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: odr_int.c,v 1.1 2003/10/27 12:21:33 adam Exp $
+ * $Id: odr_int.c,v 1.3 2004/10/15 00:19:00 adam Exp $
+ */
+/**
+ * \file odr_int.c
+ * \brief Implements ODR INTEGER codec
  */
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -33,7 +36,7 @@ int odr_integer(ODR o, int **p, int opt, const char *name)
     if (o->direction == ODR_PRINT)
     {
 	odr_prname(o, name);
-        fprintf(o->print, "%d\n", **p);
+        odr_printf(o, "%d\n", **p);
         return 1;
     }
     if (cons)
