@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2002-2004, Index Data.
+ * Copyright (c) 2002-2005, Index Data.
  * See the file LICENSE for details.
  *
- * $Id: srw.c,v 1.28 2004/10/31 19:23:22 adam Exp $
+ * $Id: srw.c,v 1.32 2005/01/11 12:05:32 adam Exp $
  */
 /**
  * \file srw.c
@@ -95,7 +95,10 @@ static int match_xsd_string_n(xmlNodePtr ptr, const char *elem, ODR o,
 #endif
     ptr = ptr->children;
     if (!ptr || ptr->type != XML_TEXT_NODE)
-        return 0;
+    {
+	*val = "";
+        return 1;
+    }
     *val = odr_strdup(o, ptr->content);
     if (len)
         *len = strlen(ptr->content);
@@ -353,6 +356,7 @@ static int yaz_srw_term(ODR o, xmlNodePtr pptr, Z_SRW_scanTerm *term,
 	term->value = 0;
 	term->numberOfRecords = 0;
 	term->displayTerm = 0;
+	term->whereInList = 0;
 	for (ptr = pptr->children; ptr; ptr = ptr->next)
 	{
 	    if (match_xsd_string(ptr, "value", o,  &term->value))
@@ -363,14 +367,18 @@ static int yaz_srw_term(ODR o, xmlNodePtr pptr, Z_SRW_scanTerm *term,
 	    else if (match_xsd_string(ptr, "displayTerm", o, 
 				      &term->displayTerm))
 		;
+	    else if (match_xsd_string(ptr, "whereInList", o, 
+				      &term->whereInList))
+		;
 	}
     }
     else if (o->direction == ODR_ENCODE)
     {
 	xmlNodePtr ptr = pptr;
 	add_xsd_string(ptr, "value", term->value);
-	add_xsd_integer(ptr, "value", term->numberOfRecords);
+	add_xsd_integer(ptr, "numberOfRecords", term->numberOfRecords);
 	add_xsd_string(ptr, "displayTerm", term->displayTerm);
+	add_xsd_string(ptr, "whereInList", term->whereInList);
     }
     return 0;
 }
@@ -605,11 +613,12 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
             (*p)->which = Z_SRW_scan_request;
             req = (*p)->u.scan_request = (Z_SRW_scanRequest *)
 		odr_malloc(o, sizeof(*req));
-	    req->database = 0;
-	    req->scanClause = 0;
-	    req->stylesheet = 0;
+	    req->query_type = Z_SRW_query_type_cql;
+	    req->scanClause.cql = 0;
 	    req->responsePosition = 0;
 	    req->maximumTerms = 0;
+	    req->stylesheet = 0;
+	    req->database = 0;
 	    
 	    for (; ptr; ptr = ptr->next)
 	    {
@@ -617,8 +626,13 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 				     &(*p)->srw_version))
 		    ;
 		else if (match_xsd_string(ptr, "scanClause", o,
-				     &req->scanClause))
+				     &req->scanClause.cql))
 		    ;
+		else if (match_xsd_string(ptr, "pScanClause", o,
+					  &req->scanClause.pqf))
+		{
+		    req->query_type = Z_SRW_query_type_pqf;
+		}
 		else if (match_xsd_integer(ptr, "responsePosition", o,
 					   &req->responsePosition))
 		    ;
@@ -782,7 +796,15 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 	    xmlSetNs(ptr, ns_srw);
 
 	    add_xsd_string(ptr, "version", (*p)->srw_version);
-	    add_xsd_string(ptr, "scanClause", req->scanClause);
+	    switch(req->query_type)
+	    {
+	    case Z_SRW_query_type_cql:
+		add_xsd_string(ptr, "scanClause", req->scanClause.cql);
+		break;
+	    case Z_SRW_query_type_pqf:
+		add_xsd_string(ptr, "pScanClause", req->scanClause.pqf);
+		break;
+	    }
 	    add_xsd_integer(ptr, "responsePosition", req->responsePosition);
 	    add_xsd_integer(ptr, "maximumTerms", req->maximumTerms);
 	    add_xsd_string(ptr, "stylesheet", req->stylesheet);
