@@ -2,7 +2,11 @@
  * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
  *
- * $Id: tcpip.c,v 1.9 2004/05/10 11:40:51 adam Exp $
+ * $Id: tcpip.c,v 1.12 2004/11/18 15:18:14 heikki Exp $
+ */
+/**
+ * \file tcpip.c
+ * \brief Implements TCP/IP + SSL COMSTACK.
  */
 
 #include <stdio.h>
@@ -24,7 +28,6 @@
 
 #include <yaz/comstack.h>
 #include <yaz/tcpip.h>
-#include <yaz/log.h>
 #include <yaz/nmem.h>
 
 #ifdef WIN32
@@ -776,7 +779,7 @@ int tcpip_get(COMSTACK h, char **buf, int *bufsize)
 #endif
 	}
 	else if (!res)
-	    return 0;
+	    return hasread;
         hasread += res;
     }
     TRC (fprintf (stderr, "  Out of read loop with hasread=%d, berlen=%d\n",
