@@ -2,9 +2,14 @@
  * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
  *
- * $Id: unix.c,v 1.4 2004/04/29 21:19:23 adam Exp $
+ * $Id: unix.c,v 1.7 2004/11/18 15:18:14 heikki Exp $
  * UNIX socket COMSTACK. By Morten Bgeskov.
  */
+/**
+ * \file unix.c
+ * \brief Implements UNIX domain socket COMSTACK
+ */
+
 #ifndef WIN32
 
 #include <stdio.h>
@@ -25,7 +30,6 @@
 
 #include <yaz/comstack.h>
 #include <yaz/unix.h>
-#include <yaz/log.h>
 #include <yaz/nmem.h>
 
 #ifndef YAZ_SOCKLEN_T
@@ -565,7 +569,7 @@ static int unix_get(COMSTACK h, char **buf, int *bufsize)
 		return -1;
 	}
 	else if (!res)
-	    return 0;
+	    return hasread;
 	hasread += res;
     }
     TRC (fprintf (stderr, "  Out of read loop with hasread=%d, berlen=%d\n",
